﻿#undef SINGLE_TEST

using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Content;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;

namespace CarFire
{
    /// <summary>
    /// This class is responsible for controlling what draws to the screen when the game is running.
    /// </summary>
    public class Display
    {
        bool playerChosen = false;
        List<Projectile> mProjectiles = new List<Projectile>();
        //List<IPlayer> mCharacters = new List<IPlayer>();
        IPlayer[] mCharacters = new IPlayer[4];
        Texture2D everything;
        Texture2D projectile1;
        Map mMap;
        int currentCenterX = 5;
        int currentCenterY = 5;
#if SINGLE_TEST
        List<Keys> mLastPressedKeys = new List<Keys>();
#endif
        public Display()
        {
            /*
            mMap = aMap;
            mCharacters = characters;
             */
        }

        /// <summary>
        /// LoadContent will be called once per game and is the place to load
        /// all of your content.
        /// </summary>
        public void LoadContent(ContentManager contentManager)
        {
            everything = contentManager.Load<Texture2D>("cs");
            projectile1 = contentManager.Load<Texture2D>("projectile");
            mMap = contentManager.Load<Map>("Maps/stable");
            Map.DefaultTile = contentManager.Load<Texture2D>("default");
            mMap.CenterCell = new Vector2(currentCenterX,currentCenterY);
            //Debugging... Spawn eight projectiles.
            //Diagonals
            /*
            mProjectiles.Add(new Projectile(mMap, everything, new Vector2(5,5), 10, 10, 300, 300));
            mProjectiles.Add(new Projectile(mMap, everything, new Vector2(-5, 5), 10, 10, 300, 300));
            mProjectiles.Add(new Projectile(mMap, everything, new Vector2(5, -5), 10, 10, 300, 300));
            mProjectiles.Add(new Projectile(mMap, everything, new Vector2(-5, -5), 10, 10, 300, 300));
            //Vertical and horizontal
            mProjectiles.Add(new Projectile(mMap, everything, new Vector2(0, 5), 10, 10, 300, 300));
            mProjectiles.Add(new Projectile(mMap, everything, new Vector2(-5, 0), 10, 10, 300, 300));
            mProjectiles.Add(new Projectile(mMap, everything, new Vector2(5, 0), 10, 10, 300, 300));
            mProjectiles.Add(new Projectile(mMap, everything, new Vector2(0, -5), 10, 10, 300, 300));
            */

            
            // TODO: use this.Content to load your game content here
        }

        /// <summary>
        /// UnloadContent will be called once per game and is the place to unload
        /// all content.
        /// </summary>
        public void UnloadContent()
        {
            // TODO: Unload any non ContentManager content here
        }

        /// <summary>
        /// Allows the game to run logic such as updating the world,
        /// checking for collisions, gathering input, and playing audio.
        /// </summary>
        /// <param name="gameTime">Provides a snapshot of timing values.</param>
        public void Update(TimeSpan timespan, GameState state)
        {
            
            //INPUT - testing input... has to be through network later
#if SINGLE_TEST
            KeyboardState keyState = Keyboard.GetState();

            List<Keys> pressedKeys = new List<Keys>();
            List<Keys> releasedKeys = new List<Keys>();

            Keys[] pressedKeysArray = keyState.GetPressedKeys();
            foreach (Keys k in pressedKeysArray)
            {
                if (!mLastPressedKeys.Contains(k)) pressedKeys.Add(k);
                else mLastPressedKeys.Remove(k);
            }

            releasedKeys = mLastPressedKeys;
            mLastPressedKeys = new List<Keys>(pressedKeysArray);
            //Just apply input for the first player
            mCharacters[0].MovePlayer(pressedKeys);
            if (pressedKeys.Contains(Keys.Enter) && !releasedKeys.Contains(Keys.Enter))
            {
                mProjectiles.Add(new Projectile(mMap, everything, new Vector2(-5,0), mCharacters[0].GridX +1, mCharacters[0].GridY+1, (int)mCharacters[0].GridX*(int)Map.PixelsToUnitSquares,(int)mCharacters[0].GridY*(int)Map.PixelsToUnitSquares));
            }
            mMap.CenterCell = new Vector2(mCharacters[0].GridX, mCharacters[0].GridY);
#endif
            
            
            for (int i = 0; i < mProjectiles.Count; i++ )
            {
                mProjectiles[i].Update(timespan);
                if (!mMap.IsCellOpen(new Point(mProjectiles[i].GridX, mProjectiles[i].GridY)))
                {
               
                    mProjectiles.RemoveAt(i);
                    i--;
                }
            }
            //Check for collisons
            for (int j = 0; j < mCharacters.Length; j++) 
            {
               
                if(mCharacters[j] != null)
                for (int i = 0; i < mProjectiles.Count; i++)
                {
                    if (mProjectiles[i].GridX == mCharacters[j].GridX && mProjectiles[i].GridY == mCharacters[j].GridY)
                    {
                        mCharacters[j].causeDamageTo(mProjectiles[i].Damage);
                        Console.WriteLine(mCharacters[j].Health);
                        mProjectiles.RemoveAt(i);
                        i--;
                    }
                }
            }
            for (int i = 0; i < 4; i++)
            {
                //If player has not selected a player yet let them select one.
                if (mCharacters[i] == null)
                {
                    if (state.keysDown[i].Contains(Keys.Enter))
                    {
                        mCharacters[i] = (new Human(mMap, "", everything, projectile1, this));
                        mCharacters[i].GridX = mMap.GetStartingPositionForPlayer(i + 1).X;
                        mCharacters[i].GridY = mMap.GetStartingPositionForPlayer(i + 1).Y;
                    }
                }
                else
                {

                    mCharacters[i].MovePlayer(state.keysDown[i]);
                    mMap.CenterCell = new Vector2(mCharacters[0].GridX, mCharacters[0].GridY);
                }
            }
        }

        /// <summary>
        /// This is called when the game should draw itself.
        /// </summary>
        /// <param name="spriteBatch">Used to draw with</param>
        public void Draw(SpriteBatch spriteBatch)
        {
            mMap.Draw(spriteBatch);
            foreach(Projectile projectile in mProjectiles)
            {
                projectile.Draw(spriteBatch);
                
            }
            for(int i = 0; i < 4; i++)//IPlayer character in mCharacters)
            {
                if(mCharacters[i] != null)
                    mCharacters[i].Draw(spriteBatch);
            } 
        }
        /*
        public void AddCharacters(IPlayer player)
        {
            mCharacters[mCharacters.Length] = player;
            //mCharacters.Add(player);
        }
         */
        public void AddProjectiles(Projectile projectile)
        {
            mProjectiles.Add(projectile);
        }
    }
}

